%%  Forecasting with Dynamic Panel Data Models
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Basic Dynamic Panel Model

clear all
clc

addpath('../../Tools')

temp_path = '../../temp/'; 
result_path = '../../results/'; 
subfolder = 'Application/';

options = optimset('Algorithm','sqp','Display','off');

%%

global N T XX YY Y0

yr0 = 2002;
yr1 = 2014;
n_sim = 1;

n_param = 3; % order: rho sigma2 ww2_lambda
lb = [0 0]'; % order: sigma2 ww2_lambda
ub = [Inf Inf]';

v_bw = .1:.1:1.9;
n_bw = length(v_bw);
bb = 1.01; 

v_K = 1:5;
n_K = length(v_K);

n_est = 4;
n_add = 0;
n_all = n_est+n_add;

%%

for T = 3:11
    
    T_smpl = (yr1-yr0)*4+4-1-T;
    
    for t_smpl = T+(1:T_smpl)
        
        yr_smpl = floor((t_smpl+1-2)/4)+yr0;
        qtr_smpl = mod((t_smpl+1-2),4)+1;
        tau_yq = yr_smpl+(qtr_smpl-1)/4;
        
        label = ['T' num2str(T) '_tau' num2str(yr_smpl) 'q' num2str(qtr_smpl) '_']
        
        %% Load the data
        
        raw = importdata([temp_path subfolder label 'data.txt']);
        
        id = unique(raw(:,1));
        N = length(id);
        
        Y = nan(T+2,N);
        
        for t = 1:(T+2)
            for i = 1:N
                Y(t,i) = raw((abs(raw(:,1)-id(i))<0.05)&(abs(raw(:,2)-(tau_yq+(-T-1+t)/4))<0.05),3);
            end
        end
        
        %% Prepare
        
        all_lambda = nan(n_est,N*n_sim);
        all_param = nan(n_param,n_est+1,n_sim);
        
        all_y = Y(2:end-1,:);
        all_x = Y(1:end-2,:);
        
        bw0 = max(N^(-1/6),log(N)^(-bb));
        
        %% Pooled OLS
        
        i_est = n_est;
        
        for i_sim = 1:n_sim
            samp_ix_xy;
            dX = XX-mean(XX(:));
            rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
            z_rho0;
            all_param(1,i_est,i_sim) = rho0;
            all_lambda(i_est,ix) = mean(z0);
        end
        
        %% Common Parameters: Post Mean (QMLE, Parametric)
        
        i_est = n_est+1;
        
        for i_sim = 1:n_sim
            
            if mod(i_sim,100) == 0
                disp([label ', QMLE, i_sim = ' num2str(i_sim)])
            end
            
            % Initialize empirical Bayesian optimization via Loss-fcn-based result
            samp_ix_xy;
            dX = XX-mean(XX);
            rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
            z_rho0;
            aux = cov(z0,Y0);
            init = [var(YY(:)-XX(:)*rho0-kron(z0',ones(T,1))); aux(1,1)-aux(1,2)^2/aux(2,2)];
            
            % QMLE
            param = fmincon(@n_log_ml_CRE,init,[],[],[],[],lb,ub,[],options);
            sigma2_0 = param(1);
            ww2_lambda0 = param(2);
            
            mX = mean(XX);
            mY = mean(YY);
            
            H = [ones(1,N); Y0];
            MH = eye(N)-(H'/(H*H'))*H;
            
            rho0 = ((XX(:)'*YY(:)-mX*mY'*T)/sigma2_0+mX*MH*mY'/(sigma2_0/T+ww2_lambda0))...
                /((sum(XX(:).^2)-sum(mX.^2)*T)/sigma2_0+sum((mX*MH).^2)/(sigma2_0/T+ww2_lambda0));
            
            z_rho0;
            mmu0 = (H*H')\(H*z0');
            
            all_param(:,i_est,i_sim) = [rho0; sigma2_0; ww2_lambda0];
            
        end
        
        %% Plug-in (QMLE)
        
        i_est_ref = n_est+1;
        i_est = n_est-1;
        plugin;
        
        %% Estimate common parameters for pseudo-out-of-sample forecasts that help with the selection of bhat or Khat 
        
        pseudo_est_CRE;
        
        %% Post Mean (QMLE, Kernel LOO, bhat)
        
        i_est_ref = n_est+1;
        i_est = n_est-3;
        basic_kernel_CRE_bhat;
        
        %% Post Mean (QMLE, Mixture, Khat)  
        
        i_est_ref = n_est+1;
        i_est = n_est-2;
        init_grp = 1;
        mixture_CRE_Khat;
        
        %% Save
        
        save([temp_path subfolder label 'results.mat'],'Y','all_param*','all_lambda','all_*hat')
        
    end
    
end