%%  Forecasting with Dynamic Panel Data Models
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Model with covariates

clear all
clc

addpath('../../Tools')

data_path = '../data/';
temp_path = '../../temp/'; 
result_path = '../../results/'; 
subfolder = 'Application/';
version_tag = '_hetero1'; 
% NOTE: Please change "version_tag" to switch between model w/ UR and model w/ UR, FFR, and spread
%   version_tag = '_hetero1': model w/ UR
%   version_tag = '_hetero3': model w/ UR, FFR, and spread

options = optimset('Algorithm','sqp','Display','off');

global N T WW XX YY Y0

yr0 = 2002;
yr1 = 2014;

load([data_path 'covs.mat'])

if strcmp(version_tag,'_hetero1')
    covs = covs(:,1);
end

n_lambda = size(covs,2)+1;
T_start = n_lambda+2;
n_param = 2+n_lambda; % order: rho sigma2 ww2_lambda(*n_heter)
lb = [0 zeros(1,n_lambda)]'; % order: sigma2 ww2_lambda(*n_heter)
ub = [Inf Inf*ones(1,n_lambda)]';

v_bw = .1:.1:1.9;
n_bw = length(v_bw);
bb = 1.01; 

v_K = 1:5;
n_K = length(v_K);

n_est = 5;

for T = T_start:11
    
    T_smpl = (yr1-yr0)*4+4-1-T;
    
    for t_smpl = T+(1:T_smpl)
        
        yr_smpl = floor((t_smpl+1-2)/4)+yr0;
        qtr_smpl = mod((t_smpl+1-2),4)+1;
        tau_yq = yr_smpl+(qtr_smpl-1)/4;
        
        label = ['T' num2str(T) '_tau' num2str(yr_smpl) 'q' num2str(qtr_smpl) '_']
        
        %% Load the data
        
        raw = importdata([temp_path subfolder label 'data.txt']);
        
        id = unique(raw(:,1));
        N = length(id);
        
        Y = nan(T+2,N);
        
        for t = 1:(T+2)
            for i = 1:N
                Y(t,i) = raw((abs(raw(:,1)-id(i))<0.05)&(abs(raw(:,2)-(tau_yq+(-T-1+t)/4))<0.05),3);
            end
        end

        %% Prepare
        
        all_lambda = nan(n_lambda,N,n_est);
        all_param = nan(n_param,n_est);
        
        YY = Y(2:end-1,:);
        XX = Y(1:end-2,:);
        Y0 = Y(1,:);
        WW = [ones(T,1) covs(t_smpl-1-T+(1:T),:)];
        
        bw0 = max(N^(-1/(n_lambda+4)),log(N)^(-bb));        
        
        %% Pooled OLS
        i_est = n_est;
        
        est_pool;
        construct_lambda0;
        
        all_param(1,i_est) = rho0;
        all_lambda(:,:,i_est) = repmat(mean(lambda0,2),1,N);
             
        %% Post Mean (QMLE, Parametric)
        
        i_est = 1;
        
        % Initialize empirical Bayesian optimization via Loss-fcn-based result
        est_within;
        construct_lambda0;
        
        aux_u = YY-rho0*XX-WW*lambda0;
        aux_Y0 = [ones(1,N); Y0];
        aux_coef = (aux_Y0*aux_Y0')\aux_Y0*lambda0';
        aux_res = lambda0'-aux_Y0'*aux_coef;
        init = [var(aux_u(:)); var(aux_res)'];
        
        % QMLE        
        param = fmincon(@n_log_ml_CRE_multilambda,init,[],[],[],[],lb,ub,[],options);
        sigma2_0 = param(1);
        ww2_lambda0 = param(end-n_lambda+1:end);
        
        H = [ones(1,N); Y0];
        MH = eye(N)-(H'/(H*H'))*H;
        
        MW = eye(T)-(WW/(WW'*WW))*WW';
        PW = (WW'*WW)\WW';
        V_lambdahat = diag(ww2_lambda0)+sigma2_0*eye(n_lambda)/(WW'*WW);
        
        rho0 = (trace(MW*YY*XX'/sigma2_0)+trace(V_lambdahat\PW*YY*MH*XX'*PW'))...
            /(trace(MW*XX*XX'/sigma2_0)+trace(V_lambdahat\PW*XX*MH*XX'*PW'));

        construct_lambda0;
        mmu0 = (H*H')\(H*lambda0');
        
        all_param(:,i_est) = [rho0; sigma2_0; ww2_lambda0];
        all_lambda(:,:,i_est) = post_mean_lambda_CRE_multilambda(lambda0,sigma2_0,mmu0,ww2_lambda0);
        
        %% Plug-in (QMLE)
        
        i_est_ref = 1;
        i_est = n_est-1;
        rho0 = all_param(1,i_est_ref);
        construct_lambda0;
        
        all_param(1,i_est) = rho0;
        all_lambda(:,:,i_est) = lambda0;
        
        %% Estimate common parameters for pseudo-out-of-sample forecasts that help with the selection of bhat or Khat 
        
        pseudo_est_CRE_multilambda;
        
        %% Post Mean (QMLE, Kernel LOO, bhat)
        
        i_est_ref = 1;
        i_est = n_est-3;
        basic_kernel_CRE_bhat_multilambda; 
        
        %% Post Mean (QMLE, Mixture, Khat)
                
        i_est_ref = 1;
        i_est = n_est-2;
        init_grp = 1;
        mixture_CRE_Khat_multilambda;
        
        %% Save
        
        save([temp_path subfolder label 'results' version_tag '.mat'],...
            'Y','all_param*','all_lambda','all_*hat')
        
    end
    
end