%% Plot
%  Percentage Change in MSE Relative to Plug-in Predictor
%  Sorted and unsorted series summarized over all samples

clear all
close all
clc

addpath('../../Tools')

data_path = '../data/';
temp_path = '../../temp/'; 
result_path = '../../results/'; 
subfolder = 'Application/'; 
version_tag = '_hetero1'; 
% NOTE: Please change "version_tag" to switch between model w/ UR and model w/ UR, FFR, and spread
%   version_tag = '_hetero1': model w/ UR
%   version_tag = '_hetero3': model w/ UR, FFR, and spread

%% Sample prepare

yr0 = 2002;
yr1 = 2014;

load([data_path 'covs.mat'])

if strcmp(version_tag,'_hetero1')
    covs = covs(:,1);
end

n_lambda = size(covs,2)+1;
T_start = n_lambda+2;
n_param = 2+n_lambda; % order: rho sigma2 ww2_lambda(*n_heter)

v_bw = .1:.1:1.9;
n_bw = length(v_bw);

v_K = 1:5;
n_K = length(v_K);

n_est = 5;

%% Plot prepare

v_estplot = [n_est-3 n_est-2 n_est n_est-1];
n_estplot = length(v_estplot);

SFont = 16;

for T = max(4,T_start):2:10
    
    T_smpl = (yr1-yr0)*4+4-1-T;
    fcst_plot = nan(T_smpl,n_estplot-1);
    
    %% Forecast
    for t_smpl = T+(1:T_smpl)
        
        yr_smpl = floor((t_smpl+1-2)/4)+yr0;
        qtr_smpl = mod((t_smpl+1-2),4)+1;
        
        label = ['T' num2str(T) '_tau' num2str(yr_smpl) 'q' num2str(qtr_smpl) '_']
        load([temp_path subfolder label 'results' version_tag '.mat'])
        N = size(Y,2);
        W1 = [1 covs(t_smpl,:)];
        
        all_z = zeros(n_estplot,N);
        for i_est = 1:n_estplot
            all_z(i_est,:) = W1*all_lambda(:,:,v_estplot(i_est));
        end
        fcst = all_z+all_param(1,v_estplot)'*Y(end-1,:);
        errs = Y(end,:)-fcst;
        
        aux = mean(errs.^2,2);
        fcst_plot(t_smpl-T,:) = (aux(1:end-1)-aux(end))/aux(end); % normalized by the plug-in predictor

    end
    
    %% Plot
    
    v_tau = (yr0+(T-1+(1:T_smpl))/4)';
    app_fmse_plot;
    close all
    
end
