%% Plot
%  Squared Forecast Error Differentials Relative To Plug-in Predictor
%  Dot plot for each sample

clear all
close all
clc

addpath('../../Tools')

data_path = '../data/';
temp_path = '../../temp/'; 
result_path = '../../results/'; 
subfolder = 'Application/';
version_tag = '_hetero1'; 
% NOTE: Please change "version_tag" to switch between model w/ UR and model w/ UR, FFR, and spread
%   version_tag = '_hetero1': model w/ UR
%   version_tag = '_hetero3': model w/ UR, FFR, and spread

%% Sample prepare

yr0 = 2002;
yr1 = 2014;

load([data_path 'covs.mat'])

if strcmp(version_tag,'_hetero1')
    covs = covs(:,1);
end

n_lambda = size(covs,2)+1;
T_start = n_lambda+2;
n_param = 2+n_lambda; % order: rho sigma2 ww2_lambda(*n_heter)

v_bw = .1:.1:1.9;
n_bw = length(v_bw);

v_K = 1:5;
n_K = length(v_K);

n_est = 5;

%% Plot prepare

v_group = 0;

v_estplot = [n_est-2 n_est n_est-1];
n_estplot = length(v_estplot);
l_estplot = {'Emp. Bayes','Pooled OLS'};

SFont = 16;

T = 6;
    
%% Forecast

for t_smpl = [21 29 41]
    
    yr_smpl = floor((t_smpl+1-2)/4)+yr0;
    qtr_smpl = mod((t_smpl+1-2),4)+1;
    
    label = ['T' num2str(T) '_tau' num2str(yr_smpl) 'q' num2str(qtr_smpl) '_']
    load([temp_path subfolder label 'results' version_tag '.mat'])
    N = size(Y,2);
    W1 = [1 covs(t_smpl,:)];
    
    all_z = zeros(n_estplot,N);
    for i_est = 1:n_estplot
        all_z(i_est,:) = W1*all_lambda(:,:,v_estplot(i_est));
    end
    fcst = all_z+all_param(1,v_estplot)'*Y(end-1,:);
    errs = Y(end,:)-fcst;
    
    ix_n = Y(end-1,:)<v_group(1);
    ix_p = ~ix_n;
    
    % plot
    F1 = figure;                                                       
    plot_errs = (errs(1:2,:).^2-errs(3,:).^2)/mean(errs(3,:).^2);
    xylim = [min(plot_errs(:)) max(plot_errs(:))];
    hold on
    scatter(plot_errs(2,ix_p),plot_errs(1,ix_p),10,'k');
    scatter(plot_errs(2,ix_n),plot_errs(1,ix_n),10,'markeredgecolor',[0 0.5 0.5],'markerfacecolor',[0 0.5 0.5]);
    plot(zeros(1,2),xylim,'k-','linewidth',0.5)
    hold off
    xlim(xylim)
    ylim(xylim)
    hline1 = refline([1,0]);
    hline2 = refline([0,0]);
    hline1.Color = 'k';
    hline2.Color = 'k';
    xlabel(l_estplot{2},'FontSize',SFont,'FontWeight','bold')
    ylabel(l_estplot{1},'FontSize',SFont,'FontWeight','bold')
    set(gca,'FontSize',SFont,'FontWeight','bold')
    set(F1,'Units','Inches','position',[0 0 3 3]);
    set(F1,'PaperOrientation','Portrait');
    set(F1,'PaperPositionMode','Auto');
    set(F1,'Clipping','off')
    print(F1,[result_path subfolder label 'err2dist_normalized' version_tag],'-dpng');
    
    xlim([-5 5])
    ylim([-5 5])
    print(F1,[result_path subfolder label 'err2dist_normalized' version_tag '_zoom5'],'-dpng');
    
end

