%% Plot
%  Predictions under Actual and Stressed Scenarios

clear all
close all
clc

addpath('../../Tools')

data_path = '../data/';
temp_path = '../../temp/'; 
result_path = '../../results/'; 
subfolder = 'Application/';
version_tag = '_hetero1'; 
% NOTE: Please change "version_tag" to switch between model w/ UR and model w/ UR, FFR, and spread
%   version_tag = '_hetero1': model w/ UR
%   version_tag = '_hetero3': model w/ UR, FFR, and spread

%% Sample prepare

yr0 = 2002;
yr1 = 2014;

load([data_path 'covs.mat'])

if strcmp(version_tag,'_hetero1')
    covs = covs(:,1);
end

n_lambda = size(covs,2)+1;
T_start = n_lambda+2;
n_param = 2+n_lambda; % order: rho sigma2 ww2_lambda(*n_heter)

v_bw = .1:.1:1.9;
n_bw = length(v_bw);

v_K = 1:5;
n_K = length(v_K);

n_est = 5;

%% Plot prepare

% As our data driven selection typically generates Khat = 1, we consider a
% gaussian prior with diagonal vcv matrix (see footnote 15).
v_estplot = [1 n_est-1 n_est]; 
n_estplot = length(v_estplot);
l_estplot = {'empb','plugin','pols'};

SFont = 16;

T = 10;
T_smpl = (yr1-yr0)*4+4-1-T;

%% Stress test

for t_smpl = [28 36]
    
    yr_smpl = floor((t_smpl+1-2)/4)+yr0;
    qtr_smpl = mod((t_smpl+1-2),4)+1;
    tau_yq = yr_smpl+(qtr_smpl-1)/4;
    
    label = ['T' num2str(T) '_tau' num2str(yr_smpl) 'q' num2str(qtr_smpl) '_']
    load([temp_path subfolder label 'results' version_tag '.mat'])
    N = size(Y,2);
    W1 = [1 covs(t_smpl,:)];
    
    raw = importdata([temp_path subfolder label 'data.txt']);
    id = unique(raw(:,1));
    assets = nan(1,N);
    for i = 1:N
        assets(i) = raw((abs(raw(:,1)-id(i))<0.05)&(abs(raw(:,2)-tau_yq)<0.05),4);
    end
    
    all_z = zeros(n_estplot,N,2); % last dim: [baseline stressed]
    for i_estplot = 1:n_estplot
        all_z(i_estplot,:,1) = W1*all_lambda(:,:,v_estplot(i_estplot));
        all_z(i_estplot,:,2) = all_z(i_estplot,:,1)+5*ones(1,n_lambda-1)*all_lambda(2:end,:,v_estplot(i_estplot));
    end
    fcst = all_z+repmat(all_param(1,v_estplot)'*Y(end-1,:),[1,1,2]);
    
    ix_big = find(assets>5e7);
    ix_small = find(assets<=5e7);
    
    for i_estplot = 1:n_estplot
        
        % plot
        if i_estplot == 2
            if t_smpl == 28
                xylim = [-31.1 41.5];
            elseif strcmp(version_tag,'_hetero3')
                xylim = [-17.9 18.3];
            else
                xylim = [-3.55 4.85];
            end
        else
            if t_smpl == 28
                xylim = [-5.65 5.65];
            else
                xylim = [-3.55 4.85];
            end
        end
        
        F1 = figure;
        hold on
        scatter(fcst(i_estplot,ix_small,1),fcst(i_estplot,ix_small,2),25,'k');
        scatter(fcst(i_estplot,ix_big,1),fcst(i_estplot,ix_big,2),50,'markeredgecolor',[0 0.5 0.5],'LineWidth',2);
        plot(zeros(1,2),xylim,'k-','linewidth',0.5)
        hold off
        xlim(xylim)
        ylim(xylim)
        hline1 = refline([1,0]);
        hline2 = refline([0,0]);
        hline1.Color = 'k';
        hline2.Color = 'k';
        xlabel('Actual','FontSize',SFont,'FontWeight','bold')
        ylabel('Stressed','FontSize',SFont,'FontWeight','bold')
        set(gca,'FontSize',SFont,'FontWeight','bold')
        set(gca,'ytick',get(gca,'xtick'))
        
        set(F1,'Units','Inches','position',[0 0 3 3]);
        set(F1,'PaperOrientation','Portrait');
        set(F1,'PaperPositionMode','Auto');
        set(F1,'Clipping','off')
        print(F1,[result_path subfolder label 'stresstest_' l_estplot{i_estplot} version_tag],'-dpng');
        
    end
    
end

close all

