#  Forecasting with Dynamic Panel Data Models
#  Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

# Load package(s)
library(moments)

# Set working directory and output paths
# https://stackoverflow.com/a/15373917
thisFile <- function() {
  cmdArgs <- commandArgs(trailingOnly = FALSE)
  needle <- "--file="
  match <- grep(needle, cmdArgs)
  if (length(match) > 0) {
    # Rscript
    return(normalizePath(sub(needle, "", cmdArgs[match])))
  } else {
    # 'source'd via R console
    return(normalizePath(sys.frames()[[1]]$ofile))
  }
}

setwd(dirname(dirname(dirname(thisFile()))))
# If error occurred in setting the working directory automatically, 
# please set it manually to the "ReplicationFiles" root directory by setwd().

dir.create("temp", showWarnings = FALSE)
dir.create("temp/Application", showWarnings = FALSE)
dir.create("results", showWarnings = FALSE)
dir.create("results/Application", showWarnings = FALSE)

# data.dir = "C:/Users/Laura/Documents/PanelData/Application_caratio/Data/" # directory for the Y9-C data, not needed as the cleaned-up dataset is provided
data.dir = "Application/data/"
temp.dir = "temp/Application/"
result.dir = "results/Application/"

# Specify the dataset
yr0 = 1987
qtr0 = 1
yr1 = 2014
qtr1 = 4
T_all = (yr1-yr0)*4+(qtr1-qtr0+1)
tag0 = paste0('_',as.character(yr0),'Q',as.character(qtr0),'_',as.character(yr1),'Q',as.character(qtr1))

#===========================================================================================================
# 1. read data

# NOTE: not needed as the cleaned-up dataset is provided

# vars = c("BHCK4074",
#          "BHCK4079",
#          "BHCK4093","BHCKC216","BHCKC232",
#          "BHCK3368")
# vars_co = vars[1:5]
# vars_loan = vars[6]
#
# for(t in 1:T_all) {
#
#   yr = (t+qtr0-2)%/%4+yr0
#   qtr = (t+qtr0-2)%%4+1
#
#   yyyymm = yr*100+qtr*3
#   yymm = substr(as.character(yyyymm), start=3, stop=6)
#   print(yymm)
#
#   temp_q = read.table(paste0(data.dir,"bhcf",yymm,".txt"), sep="^", comment.cha="", header=TRUE, quote="", na.strings="--------", as.is=TRUE)
#   names(temp_q) = toupper(names(temp_q))
#
#   temp_q[,vars[!(vars %in% colnames(temp_q))]] = NA
#
#   temp_q = subset(temp_q,
#                   select=c(BHCK4074,
#                            BHCK4079,
#                            BHCK4093,BHCKC216,BHCKC232,
#                            BHCK3368,
#                            RSSD9001))
#
#   temp_q =  temp_q[!duplicated(temp_q$RSSD9001, fromLast = TRUE),]
#   temp_q =  temp_q[!is.na(temp_q$RSSD9001),]
#
#   # chargeoffs: NA ~> 0
#   aux_q = temp_q[,vars_co]
#   aux_q[is.na(aux_q)] = 0
#   temp_q[,vars_co] = aux_q
#
#   temp_q$yyyy = yr
#   temp_q$q = qtr
#   temp_q$y.q = yr+(qtr-1)/4
#
#   temp_q_aux = temp_q
#
#   if (t >= 2) {
#
#     # loans: previous quarter
#     id_q = unique(temp_q$RSSD9001)
#     id_q0 = unique(temp_q0$RSSD9001)
#
#     for (ind_id in id_q){
#       if (!is.element(ind_id,id_q0)){
#         temp_q[temp_q$RSSD9001==ind_id,vars_loan] = NA
#       } else {
#         temp_q[temp_q$RSSD9001==ind_id,vars_loan] = temp_q0[temp_q0$RSSD9001==ind_id,vars_loan]
#       }
#     }
#
#     # chargeoffs: ytd data ~> flow data
#     if (qtr > 1) {
#       for (ind_id in id_q){
#         if (!is.element(ind_id,id_q0)){
#           temp_q[temp_q$RSSD9001==ind_id,vars_co] = NA
#         } else {
#           temp_q[temp_q$RSSD9001==ind_id,vars_co] = temp_q[temp_q$RSSD9001==ind_id,vars_co]-temp_q0[temp_q0$RSSD9001==ind_id,vars_co]
#         }
#       }
#     }
#
#     if (qtr == 1 | t == 2) {
#       temp_y = temp_q
#     } else {
#       temp_y = rbind(temp_y,temp_q)
#     }
#
#   }
#
#   temp_q0 = temp_q_aux
#
#   if (qtr == 4 | t == T_all) {
#
#     attach(temp_y)
#     temp_y$nii = 400*BHCK4074/BHCK3368
#     temp_y$onii = 400*BHCK4079/BHCK3368
#     temp_y$onie = 400*(BHCK4093-BHCKC216-BHCKC232)/BHCK3368
#     detach(temp_y)
#
#     if (t <= 4) {
#       bank = temp_y
#     } else {
#       bank = rbind(bank,temp_y)
#     }
#   }
#
# }
#
# bank = bank[order(bank$yyyy,bank$q,bank$RSSD9001),]
#
# save(bank, file=paste0(data.dir,"bank",tag0,".RData"))
load(file=paste0(data.dir,"bank",tag0,".RData"))

#===========================================================================================================
#2. sample selection

attach(bank)
bank$ppnr=nii+onii-onie
detach(bank)

v_var = "ppnr"
n_var = length(v_var)

v_stats = c("mean","median")
n_stats = length(v_stats)

tol = 10
yr00 = 2002

n_smpl = ((yr1-yr00)*4+qtr1-qtr0-(3+11)/2)*9
n_output = matrix(NA,n_smpl,8)
colnames(n_output) = c("T","tau",paste0("n",c(0:5)))

stats_output = matrix(NA,n_smpl,11)
colnames(stats_output) = c("T","tau","mean","min","q25","median","q75","max","std","skew","kurt")

count = 0

for (TT in 3:11) {

  T2 = TT+2
  T_smpl = (yr1-yr00)*4+qtr1-qtr0-T2+2

  for (t_smpl in TT+(1:T_smpl)) {
    yr_smpl = (t_smpl+qtr0-2)%/%4+yr00
    qtr_smpl = (t_smpl+qtr0-2)%%4+1
    tau_yq = yr_smpl+(qtr_smpl-1)/4

    data = bank[(bank$y.q>=tau_yq-TT/4-0.01)&(bank$y.q<=tau_yq+1/4+0.01),c("RSSD9001","BHCK3368","y.q","ppnr")]
    n0 = length(unique(data$RSSD9001))

    # Step 1: Delete banks for which total assets are missing for all time periods in the sample.
    data = data[is.finite(data$BHCK3368),]
    n1 = length(unique(data$RSSD9001))

    # Step 2: Compute average non-missing total assets and eliminate BHCs with average assets below 500 million.
    # (Inflation, industry consolidation, and normal asset growth of BHCs led to an increase in the asset-size threshold for filing the FR Y-9C from $150 million to $500 million, effective with the March 2006, report date.)
    aux2 = aggregate(BHCK3368~RSSD9001, data = data, function(x) mean(x,na.rm=TRUE))
    ix2 = aux2$BHCK3368 > 500000
    id2 = aux2[as.logical(ix2),1]
    data = data[data$RSSD9001%in%id2,]
    n2 = sum(ix2)

    # Step 3: Delete a bank if one or more PPNR subcomponents are missing (or infinite) for at least one period during the sample.
    aux3 = aggregate(ppnr~RSSD9001, data = data, function(x) sum(is.finite(x)))
    ix3 = aux3$ppnr == T2
    id3 = aux3[as.logical(ix3),1]
    data = data[data$RSSD9001%in%id3,]
    n3 = sum(ix3)

    # Step 4: |temporal mean - temporal median| < 10
    aux_list = list()

    for (i_stats in 1:n_stats) {
      aux_list[[i_stats]] = eval(parse(text=paste0('aggregate(ppnr~RSSD9001, data = data, ', v_stats[i_stats], ')')))
    }

    aux_stats = merge(aux_list[[1]],aux_list[[2]],by="RSSD9001")
    ix4 = abs(aux_stats[,2]-aux_stats[,3])<tol
    n4 = sum(ix4)

    # Step 5: Keep institution $i$ if $\text{Dev}_{it}$'s fall within {[}$q_{0.5}$ -/+ ($q_{0.975}$
    dev = merge(data[,c("RSSD9001","ppnr")],aux_stats[ix4,1:2],by="RSSD9001")
    dev[,2] = dev[,2]-dev[,3]
    dev = dev[is.finite(dev[,2]),-3]
    q_dev =  quantile(dev[,2], probs = c(.025,.5,.975))
    rg_dev = q_dev[2]+(q_dev[3]-q_dev[1])*c(-1,1)
    aux_dev = aggregate(ppnr~RSSD9001, data = dev, function(x) prod(x>rg_dev[1] & x<rg_dev[2]))
    ix5 = aux_dev[,2]
    id5 = aux_dev[as.logical(ix5),1]
    data = data[data$RSSD9001%in%id5,]
    n5 = sum(ix5)

    write.table(data[,c("RSSD9001","y.q","ppnr","BHCK3368")],
                file = paste0(temp.dir,"T",as.character(TT),"_tau",as.character(yr_smpl),"q",as.character(qtr_smpl),"_data.txt"),
                row.names=FALSE, col.names=FALSE)

    # outputs
    count = count+1
    n_output[count,] = c(TT,tau_yq,n0,n1,n2,n3,n4,n5)
    stats_output[count,1:2] = c(TT,tau_yq)
    attach(data)
    stats_output[count,3] = mean(ppnr)
    stats_output[count,4:8] = quantile(ppnr)
    stats_output[count,9] = sd(ppnr)
    stats_output[count,10] = skewness(ppnr)
    stats_output[count,11] = kurtosis(ppnr)
    detach(data)
  }
}

stats_output_aux = stats_output
stats_output_aux[,3:11] = round(stats_output_aux[,3:11],digits=2)

#2.5 selection setup
write.csv(n_output, file = paste0(result.dir,"n_output.csv"),row.names=FALSE)
write.csv(stats_output_aux, file = paste0(result.dir,"stats_output.csv"),row.names=FALSE)