%%  Forecasting with Dynamic Panel Data Models
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Simulations: design 1

clear all
clc

addpath('../../Tools')

mkdir ../../temp/Experiment1
mkdir ../../results/Experiment1

temp_path = '../../temp/'; 
result_path = '../../results/';
subfolder = 'Experiment1/';

options = optimset('Algorithm','sqp','Display','off');

%% DGP

global N T XX YY Y0

v_N = [100 1000];
n_N = length(v_N);
T = 4;
n_sim = 1000;

sigma2 = 1;
v_ww2_lambda = [1 .1 .01 0];
l_ww2_lambda = {'100';'010';'001';'000'};
n_ww2_lambda = length(v_ww2_lambda);

n_param = 3; % order: rho sigma2 ww2_lambda
lb = [0 0]'; % order: sigma2 ww2_lambda
ub = [Inf Inf]';

v_bw = 1:.1:3;
n_bw = length(v_bw);
bb = 1.01; 
v_e0 = [0.2 0.7];
n_e0 = length(v_e0);

v_K = 1:5;
n_K = length(v_K);

n_est = 5+n_bw;
l_est = cell(n_est-1,1);
l_est([1 end-2:end]) = {'Kernel bhat';'Mixture Khat';'Plug-in';'Pooled OLS'};
l_est(2:n_bw+1) = strcat('Kernel b=',cellstr(num2str(v_bw')));
l_est = char(l_est);

N_lsel = 10000;

rho = 0.8;
mmu = [0 0]';
kk = 2; % lambda ~ Gamma(kk,ttheta)

for i_N = 1:n_N
    
    N = v_N(i_N);
    bw0 = max(N^(-1/5),log(N)^(-bb));
    
    for i_ww2_lambda = 1:n_ww2_lambda
        
        label = ['exp1_N' num2str(N) '_gamma' l_ww2_lambda{i_ww2_lambda}]
        
        %% Parameters
        
        ww2_lambda = v_ww2_lambda(i_ww2_lambda);
        ttheta = sqrt(ww2_lambda/kk); % lambda ~ Gamma(kk,ttheta)
        
        %% Data
        
        RandStream.setGlobalStream(RandStream('mcg16807','Seed',1));
        Y = zeros(T+2,N*n_sim);
        Y(1,:) = randn(1,N*n_sim);
        
        lambda = gamrnd(kk,ttheta,[1,N*n_sim]);
        
        u = randn(T+1,N*n_sim);
        for t = 1:T+1
            Y(t+1,:) = rho*Y(t,:)+lambda+u(t,:)*sqrt(sigma2);
        end
        
        %% Prepare
        
        all_lambda = zeros(n_est,N*n_sim);
        all_param = nan(n_param,n_est,n_sim);
        
        all_y = Y(2:end-1,:);
        all_x = Y(1:end-2,:);
        all_z = mean(all_y)-rho*mean(all_x);
        
        %% Oracle
        
        i_est = 1;
        
        all_param(1:2,i_est,:) = repmat([rho; sigma2],[1 1 n_sim]);
        
        if ww2_lambda ~= 0
            
            if i_N == 1
                v_aux = nan(1,N_lsel);
            end
            
            for i = 1:N*n_sim
                
                if mod(i,100*N) == 0
                    disp([label ', i_est = ' num2str(i_est) ', i = ' num2str(i)])
                end
            
                m0 = integral(@(x) normpdf(all_z(i),x,sqrt(sigma2/T)).*gampdf(x,kk,ttheta),0,Inf);
                m1 = integral(@(x) x.*normpdf(all_z(i),x,sqrt(sigma2/T)).*gampdf(x,kk,ttheta),0,Inf);
                
                all_lambda(i_est,i) = m1/m0;
                
                if i_N == 1
                    if i <= N_lsel
                        m2 = integral(@(x) x.^2.*normpdf(all_z(i),x,sqrt(sigma2/T)).*gampdf(x,kk,ttheta),0,Inf);
                        v_aux(i) = m2/m0-(m1/m0)^2;
                    end
                end
                
            end
            
            if i_N == 1
                post_var_lambda =  mean(v_aux);
            end
            
        else
            
            all_lambda(i_est,:) = 0;
            post_var_lambda = 0;
            
        end
        
        %% Pooled OLS
        
        i_est = n_est;
        
        for i_sim = 1:n_sim
            samp_ix_xy;
            dX = XX-mean(XX(:));
            rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
            z_rho0;
            all_param(1,i_est,i_sim) = rho0;
            all_lambda(i_est,ix) = mean(z0);
        end
        
        %% Common Parameters: Post Mean (QMLE, Parametric)
        
        i_est = n_est+1;
        
        for i_sim = 1:n_sim
            
            if mod(i_sim,100) == 0
                disp([label ', QMLE, i_sim = ' num2str(i_sim)])
            end
            
            % Initialize empirical Bayesian optimization via Loss-fcn-based result
            samp_ix_xy;
            dX = XX-mean(XX);
            rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
            z_rho0;
            aux = cov(z0,Y0);
            init = [var(YY(:)-XX(:)*rho0-kron(z0',ones(T,1))); var(z0)];
            
            % QMLE
            param = fmincon(@n_log_ml_RE,init,[],[],[],[],lb,ub,[],options);
            sigma2_0 = param(1);
            ww2_lambda0 = param(2);
            
            mX = mean(XX);
            mY = mean(YY);
            
            rho0 = ((XX(:)'*YY(:)-mX*mY'*T)/sigma2_0+(mX-mean(mX))*(mY-mean(mY))'/(sigma2_0/T+ww2_lambda0))...
                /((sum(XX(:).^2)-sum(mX.^2)*T)/sigma2_0+sum((mX-mean(mX)).^2)/(sigma2_0/T+ww2_lambda0));
            
            z_rho0;
            mmu0 = mean(z0);
            
            all_param(:,i_est,i_sim) = [rho0; sigma2_0; ww2_lambda0];
            
        end
        
        %% Plug-in (QMLE)
        
        i_est_ref = n_est+1;
        i_est = n_est-1;
        plugin;
        
        %% Estimate common parameters for pseudo-out-of-sample forecasts that help with the selection of bhat or Khat
        
        pseudo_est_RE;
        
        %% Post Mean (QMLE, Kernel LOO, bhat)
        
        i_est_ref = n_est+1;
        i_est = 2;
        basic_kernel_RE_bhat;
        
        %% Post Mean (QMLE, Kernel LOO)
        
        i_est_ref = n_est+1;
        
        for i_bw = 1:n_bw
            i_est = 2+i_bw;
            basic_kernel_RE;
        end
        
        %% Post Mean (QMLE, Mixture, Khat)
        
        i_est_ref = n_est+1;
        i_est = n_est-2;
        init_grp = 1;
        mixture_RE_Khat;
        
        %% Save
        
        save([temp_path subfolder label '.mat'],'Y','lambda','post_var_lambda','all_param*','all_lambda','all_*hat')
        
        %% Table - forecast error
        
        all_param(:,n_est+1,:) = [];
        fcst = all_lambda+kron(squeeze(all_param(1,:,:)),ones(1,N)).*Y(end-1,:);
        errs = Y(end,:)-fcst;
        
        % Table
        table_fcst_err_aux = sum(errs.^2,2)/n_sim;
        table_fcst_err = ...
            (table_fcst_err_aux(2:end) - table_fcst_err_aux(1))./(N*post_var_lambda+N.^v_e0);
        
        %% Output table
        
        l_col = strcat('e0=',cellstr(num2str(v_e0')));
        n_col = n_e0;
        
        fileID = fopen([result_path subfolder 'table_' label '.txt'],'w');
        
        fprintf(fileID,'%s',repmat(' ',1,size(l_est,2)));
        fprintf(fileID,' %8s',l_col{1:end});
        fprintf(fileID,' \r\n');
        
        for i_est = 1:n_est-1
            fprintf(fileID,['%s' repmat(' %8.3f',1,n_col) ' \r\n'],l_est(i_est,:),table_fcst_err(i_est,:));
        end
        
        fclose(fileID);
        
        
        %% Plot
        
        if N == 1000 && ismember(i_ww2_lambda,[1 4])
            main_design1_plot;
        end
        
    end
    
end
