%% Squared Forecast Error Loss as a Function of \hat\lambda_i

close all

h1 = figure(1);
set(h1,'Units','Inches','position',[0 0 3 3])
set(h1,'PaperOrientation','portrait');
set(h1,'PaperPositionMode','Auto');
set(h1,'Clipping','off')
left_color = [0 0 0];
right_color = [0 .5 .5];
set(h1,'defaultAxesColorOrder',[left_color; right_color]);

SFont = 16;

v_estplot = 2;
n_estplot = length(v_estplot);
eststr = cell(n_estplot,1);
eststr{1}= 'bhatloo';

n_bin = 500;
i_est_ref = n_est-1;
agg_lambda_hat = quantile(all_lambda(i_est_ref,:),((1:n_bin)-0.5)/n_bin);
lambda_aux = [min(all_lambda(i_est_ref,:))-1 quantile(all_lambda(i_est_ref,:),(1:(n_bin-1))/n_bin) max(all_lambda(i_est_ref,:))+1];
agg_est = nan(n_estplot,n_bin);
xxlim = [agg_lambda_hat(1)-0.2 agg_lambda_hat(end)+0.2];

for i_bin = 1:n_bin
    ix = find(all_lambda(i_est_ref,:)>lambda_aux(i_bin) & all_lambda(i_est_ref,:)<=lambda_aux(i_bin+1));
    agg_est(:,i_bin) = mean(errs(v_estplot,ix).^2,2);
end

yylim = [min(agg_est(:)) max(agg_est(:))];

figure(1)
for i_est = 1:n_estplot
    yyaxis right
    [ff,xx] = ksdensity(all_z,linspace(xxlim(1),xxlim(2),100));
    plot(xx,ff,':','linewidth',2);
    ylim([0 max(ff*2)])
    yyaxis left
    scatter(agg_lambda_hat,agg_est(i_est,:),10,'k')
    xlim(xxlim)
    ylim(yylim)
    set(gca,'FontSize',SFont,'FontWeight','bold')
    print(h1,[result_path subfolder label '_mse_' eststr{i_est}],'-dpng');
end

%% Tweedie Corrections

close all

h1 = figure(1);
set(h1,'Units','Inches','position',[0 0 3 3])
set(h1,'PaperOrientation','portrait');
set(h1,'PaperPositionMode','Auto');
set(h1,'Clipping','off')
left_color = [0 0 0];
right_color = [0 .5 .5];
set(h1,'defaultAxesColorOrder',[left_color; right_color]);

all_zhat = nan(2,N,n_sim);
all_tweedie = nan(n_estplot+1,N,n_sim);

for i_sim = 1:n_sim
    samp_ix;
    all_zhat(:,:,i_sim) = [all_z(ix); all_lambda(i_est_ref,ix)];
    all_tweedie(:,:,i_sim) = all_lambda([1 v_estplot],ix)-[all_z(ix); repmat(all_lambda(i_est_ref,ix),n_estplot,1)];
end

[zhat0,ix_zhat0] = sort(reshape(all_zhat(1,:,:),1,[]));
aux_tweedie = reshape(all_tweedie(1,:,:),1,[]);
tweedie0 = aux_tweedie(ix_zhat0);

[zhat1,ix_zhat1] = sort(squeeze(all_zhat(2,:,:)));
tweedie1 = nan(n_estplot,N,n_sim);
for i_sim = 1:n_sim
    tweedie1(:,:,i_sim) = all_tweedie(2:end,ix_zhat1(:,i_sim),i_sim);
end

ix_zhat1 = find(zhat1(:)>=xxlim(1)& zhat1(:)<=xxlim(2));
yy_aux = nan(n_estplot,2);
for i_est = 1:n_estplot
   tweedie1_aux = reshape(tweedie1(i_est,:,:),[],1);
   yy_aux(i_est,:) = [min(tweedie1_aux(ix_zhat1)) max(tweedie1_aux(ix_zhat1))];
end

yylim = [min(yy_aux(:,1)) max(yy_aux(:,2))];

figure(1)
for i_est = 1:n_estplot
    yyaxis right
    [ff,xx] = ksdensity(all_z,linspace(xxlim(1),xxlim(2),100));
    plot(xx,ff,':','linewidth',2);
    ylim([0 max(ff*2)])
    yyaxis left
    hold on
    for i_sim = 1:n_sim
        patchline(zhat1(:,i_sim),squeeze(tweedie1(i_est,:,i_sim)),'linestyle','-','edgecolor',[.7 .7 .7],'linewidth',1,'edgealpha',0.05);
    end
    plot(zhat0,tweedie0,'k-','linewidth',2);
    hold off
    xlim(xxlim)
    ylim(yylim)
    set(gca,'FontSize',SFont,'FontWeight','bold')
    print(h1,[result_path subfolder label '_tweedie_' eststr{i_est}],'-dpng');
end

 