%%  Forecasting with Dynamic Panel Data Models
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Simulations: design 1, updated with NP MLE predictor

clear all
clc

addpath('../../Tools')

temp_path = '../../temp/'; 
result_path = '../../results/';
current_dir = pwd;
tools_path = [strrep(current_dir(1:end-18),'\','/') 'Tools/']; 
subfolder = 'Experiment1/';

options = optimset('Algorithm','sqp','Display','off');

%% DGP

global N T XX YY Y0

v_N = [100 1000];
n_N = length(v_N);
T = 4;
n_sim = 1000;

sigma2 = 1;
v_ww2_lambda = [1 .1 .01 0];
l_ww2_lambda = {'100';'010';'001';'000'};
n_ww2_lambda = length(v_ww2_lambda);

n_param = 3; % order: rho sigma2 ww2_lambda
lb = [0 0]'; % order: sigma2 ww2_lambda
ub = [Inf Inf]';

v_bw = 1:.1:3;
n_bw = length(v_bw);
bb = 1.01; 
v_e0 = [0.2 0.7];
n_e0 = length(v_e0);

v_K = 1:5;
n_K = length(v_K);

n_est = 6+n_bw;
l_est = cell(n_est-1,1);
l_est([1 end-3:end]) = {'Kernel bhat';'Mixture Khat';'Plug-in';'Pooled OLS';'NP MLE'};
l_est(2:n_bw+1) = strcat('Kernel b=',cellstr(num2str(v_bw')));
l_est = char(l_est);

N_lsel = 10000;

rho = 0.8;
mmu = [0 0]';
kk = 2; % lambda ~ Gamma(kk,ttheta)

for i_N = 1:n_N
    
    N = v_N(i_N);
    
    for i_ww2_lambda = 1:n_ww2_lambda
        
        label = ['exp1_N' num2str(N) '_gamma' l_ww2_lambda{i_ww2_lambda}]
        load([temp_path subfolder label '.mat'])
        
        %% Prepare
        
        all_y = Y(2:end-1,:);
        all_x = Y(1:end-2,:);
        all_param(3:end,n_est,:) = nan;
        
        %% Post Mean (QMLE, NP MLE)

        i_est_ref = n_est;
        i_est = n_est;
        gk;
        
        %% Save
        
        save([temp_path subfolder label '_update_npmle.mat'],'Y','lambda','post_var_lambda','all_param*','all_lambda','all_*hat')
        
        %% Table - forecast error
        
        fcst = all_lambda+kron(squeeze(all_param(1,:,:)),ones(1,N)).*Y(end-1,:);
        errs = Y(end,:)-fcst;
        
        % Table
        table_fcst_err_aux = sum(errs.^2,2)/n_sim;
        table_fcst_err = ...
            (table_fcst_err_aux(2:end) - table_fcst_err_aux(1))./(N*post_var_lambda+N.^v_e0);
                
       %% Output table
        
        l_col = strcat('e0=',cellstr(num2str(v_e0')));
        n_col = n_e0;
        
        fileID = fopen([result_path subfolder 'table_' label '_update_npmle.txt'],'w');
        
        fprintf(fileID,'%s',repmat(' ',1,size(l_est,2)));
        fprintf(fileID,' %8s',l_col{1:end});
        fprintf(fileID,' \r\n');
        
        for i_est = 1:n_est-1
            fprintf(fileID,['%s' repmat(' %8.3f',1,n_col) ' \r\n'],l_est(i_est,:),table_fcst_err(i_est,:));
        end
        
        fclose(fileID);
        
    end
    
end
