%%  Forecasting with Dynamic Panel Data Models
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Simulations: design 2

clear all
clc

addpath('../../Tools')

mkdir ../../temp/Experiment2
mkdir ../../results/Experiment2

temp_path = '../../temp/'; 
result_path = '../../results/';
subfolder = 'Experiment2/';

options = optimset('Algorithm','sqp','Display','off');

%% DGP

global N T XX YY Y0

N = 1000; 
T = 4;
n_sim = 1000; 

sigma2 = 1;
rho = 0.8;
mu_lambda = 1;
w2_lambda = 1;
v_ddelta = [.05 .1 .3];
l_ddelta = {'05','10','30'};
n_ddelta = length(v_ddelta);

ww2_lambda = 1/((1-rho^2)/sigma2/(1-rho)^2+1/w2_lambda);
mmu = ww2_lambda*[mu_lambda/w2_lambda (1-rho^2)/sigma2/(1-rho)]';
mu_y0 = mu_lambda/(1-rho);
w2_y0 = sigma2/(1-rho^2)+w2_lambda/(1-rho)^2;
var_z = sigma2/T+ww2_lambda;

n_param = 3; % order: rho sigma2 ww2_lambda
lb = [0 0]'; % order: sigma2 ww2_lambda
ub = [Inf Inf]';

v_bw = .1:.1:1.9;
n_bw = length(v_bw);
bb = 1.01; 
bw0 = max(N^(-1/6),log(N)^(-bb));
e0 = 0.7;

v_K = 1:5;
n_K = length(v_K);

n_est = 5;
table_fcst_err = zeros(n_est-1,n_ddelta);

%% Data - I

RandStream.setGlobalStream(RandStream('mcg16807','Seed',1));
lambda_base = randn(1,N*n_sim);
Y0_base = randn(1,N*n_sim);
Y = zeros(T+2,N*n_sim);
u_base = randn(T+1,N*n_sim);

Y(1,:) = mu_y0+Y0_base*sqrt(w2_y0);

%%

for i_ddelta = 1:3
    
    ddelta = v_ddelta(i_ddelta);
    label = ['exp2_delta' l_ddelta{i_ddelta}]
    
    %% Parameters
    
    mu_pos = mmu(1)+ddelta+(mmu(2)+ddelta)*Y(1,:);
    mu_neg = mmu(1)-ddelta+(mmu(2)-ddelta)*Y(1,:);
    
    %% Data - II
    
    lambda = lambda_base*sqrt(ww2_lambda)+mu_neg+(mu_pos-mu_neg).*(rand(1,N*n_sim)>0.5);
    
    for t = 1:T+1
        Y(t+1,:) = rho*Y(t,:)+lambda+u_base(t,:)*sqrt(sigma2);
    end
    
    %% Prepare
    
    all_lambda = nan(n_est,N*n_sim);
    all_param = nan(n_param,n_est+1,n_sim);
    
    all_y = Y(2:end-1,:);
    all_x = Y(1:end-2,:);
    all_z = mean(all_y)-rho*mean(all_x);
    
    %% Oracle
    
    i_est = 1;
    
    all_param(:,i_est,:) = repmat([rho; sigma2; ww2_lambda],[1 1 n_sim]);
    dnorm_pos = normpdf(all_z,mu_pos,sqrt(var_z));
    dnorm_neg = normpdf(all_z,mu_neg,sqrt(var_z));
    all_lambda(i_est,:) = (all_z*ww2_lambda+(mu_pos.*dnorm_pos+mu_neg.*dnorm_neg)./(dnorm_pos+dnorm_neg)*sigma2/T)/var_z;
	post_var_lambda = sigma2/T*(1+sigma2/T*(dnorm_pos.*dnorm_neg.*(mu_pos-mu_neg).^2/var_z./(dnorm_pos+dnorm_neg).^2-1)/var_z);
    
     %% Pooled OLS
     
    i_est = n_est;
    
    for i_sim = 1:n_sim
        samp_ix_xy;
        dX = XX-mean(XX(:));
        rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
        z_rho0;
        all_param(1,i_est,i_sim) = rho0;
        all_lambda(i_est,ix) = mean(z0);
    end
   
    %% Common Parameters: Post Mean (QMLE, Parametric)
    
    i_est = n_est+1;
    
    for i_sim = 1:n_sim
        
        if mod(i_sim,100) == 0
            disp([label ', QMLE, i_sim = ' num2str(i_sim)])
        end
        
        % Initialize empirical Bayesian optimization via Loss-fcn-based result
        samp_ix_xy;
        dX = XX-mean(XX);
        rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
        z_rho0;        
        aux = cov(z0,Y0);
        init = [var(YY(:)-XX(:)*rho0-kron(z0',ones(T,1))); aux(1,1)-aux(1,2)^2/aux(2,2)];
             
        % QMLE
        param = fmincon(@n_log_ml_CRE,init,[],[],[],[],lb,ub,[],options);
        sigma2_0 = param(1);
        ww2_lambda0 = param(2);
        
        mX = mean(XX);
        mY = mean(YY);
        
        H = [ones(1,N); Y0];
        MH = eye(N)-(H'/(H*H'))*H;
        
        rho0 = ((XX(:)'*YY(:)-mX*mY'*T)/sigma2_0+mX*MH*mY'/(sigma2_0/T+ww2_lambda0))...
            /((sum(XX(:).^2)-sum(mX.^2)*T)/sigma2_0+sum((mX*MH).^2)/(sigma2_0/T+ww2_lambda0));
        
        z_rho0;
        mmu0 = (H*H')\(H*z0');
        
        all_param(:,i_est,i_sim) = [rho0; sigma2_0; ww2_lambda0];
        
    end
    
    %% Plug-in (QMLE)
    
    i_est_ref = n_est+1;
    i_est = n_est-1;
    plugin;
    
    %% Estimate common parameters for pseudo-out-of-sample forecasts that help with the selection of bhat or Khat 
    
    pseudo_est_CRE;
    
    %% Post Mean (QMLE, Kernel LOO, bhat)
    
    i_est_ref = n_est+1;
    i_est = n_est-3;
    basic_kernel_CRE_bhat;
    
    %% Post Mean (QMLE, Mixture, Khat)
    
    i_est_ref = n_est+1;
    i_est = n_est-2;
    init_grp = 1;
    mixture_CRE_Khat;
    
    %% Save
    
    save([temp_path subfolder label '.mat'],'Y','lambda','post_var_lambda','all_param*','all_lambda','all_*hat')
    
    %% Forecast error
    
    all_param(:,n_est+1,:) = [];
    fcst = all_lambda+kron(squeeze(all_param(1,:,:)),ones(1,N)).*Y(end-1,:);
    errs = Y(end,:)-fcst;
    
    % Table
    table_fcst_err_aux = sum(errs.^2,2)/n_sim;
    table_fcst_err(:,i_ddelta) = ...
        (table_fcst_err_aux(2:end) - table_fcst_err_aux(1))/(N*mean(post_var_lambda)+N^e0);
    
end

%% Output table

l_col = strcat('delta',l_ddelta);
n_col = n_ddelta;
l_est = char({'Kernel bhat';'Mixture Khat';'Plug-in';'Pooled OLS'});

fileID = fopen([result_path subfolder 'table.txt'],'w');

fprintf(fileID,'%s',repmat(' ',1,size(l_est,2)));
fprintf(fileID,' %8s',l_col{1:end});
fprintf(fileID,' \r\n');

for i_est = 1:n_est-1
    fprintf(fileID,['%s' repmat(' %8.3f',1,n_col) ' \r\n'],l_est(i_est,:),table_fcst_err(i_est,:));
end

fclose(fileID);
