%%  Forecasting with Dynamic Panel Data Models
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Simulations: design 3

clear all
clc

addpath('../../Tools')

mkdir ../../temp/Experiment3
mkdir ../../results/Experiment3

temp_path = '../../temp/'; 
result_path = '../../results/'; 
subfolder = 'Experiment3/';

options = optimset('Algorithm','sqp','Display','off');

%% DGP

global N T XX YY Y0

N = 1000; 
T = 4;
n_sim = 1000; 

sigma2 = 1;
rho = 0.8;
mu_lambda = 1;
w2_lambda = 1;
v_mix = [0 0 4 1/4 1/5; 2 -1/4 1/2 1/2 1/9];
n_mix = size(v_mix,1);
l_mix = {'scalemix','locmix'};

ww2_lambda = 1/((1-rho^2)/sigma2/(1-rho)^2+1/w2_lambda);
mmu = ww2_lambda*[mu_lambda/w2_lambda (1-rho^2)/sigma2/(1-rho)]';
mu_y0 = mu_lambda/(1-rho);
w2_y0 = sigma2/(1-rho^2)+w2_lambda/(1-rho)^2;
var_z = sigma2/T+ww2_lambda;

ddelta = .1;

n_param = 3; % order: rho sigma2 ww2_lambda
lb = [0 0]'; % order: sigma2 ww2_lambda
ub = [Inf Inf]';

v_bw = .1:.1:1.9;
n_bw = length(v_bw);
bb = 1.01; 
bw0 = max(N^(-1/6),log(N)^(-bb));
e0 = 0.7;

v_K = 1:5;
n_K = length(v_K);

n_est = 5;
table_fcst_err = zeros(n_est-1,n_mix);

%% Data - I

RandStream.setGlobalStream(RandStream('mcg16807','Seed',1));
lambda_base = randn(1,N*n_sim);
Y0_base = randn(1,N*n_sim);
Y = zeros(T+2,N*n_sim);
u_base = randn(T+1,N*n_sim);
v_base = rand(T+1,N*n_sim);

Y(1,:) = mu_y0+Y0_base*sqrt(w2_y0);

%% Parameters

mu_pos = mmu(1)+ddelta+(mmu(2)+ddelta)*Y(1,:);
mu_neg = mmu(1)-ddelta+(mmu(2)-ddelta)*Y(1,:);
    
%% Data - II

lambda = lambda_base*sqrt(ww2_lambda)+mu_neg+(mu_pos-mu_neg).*(rand(1,N*n_sim)>0.5);

%%

for i_mix = 1:n_mix
    
    label = ['exp3_' l_mix{i_mix}]
    
    %% Parameters
    
    mu_u = v_mix(i_mix,1:2);
    gamma2_u = v_mix(i_mix,3:4);
    p_u = v_mix(i_mix,5);
    
    %% Data - III
    
    v = v_base < p_u;
    u = u_base.*sqrt(gamma2_u(1)*(v==1)+gamma2_u(2)*(v==0))+mu_u(1)*(v==1)+mu_u(2)*(v==0);
            
    for t = 1:T+1
        Y(t+1,:) = rho*Y(t,:)+lambda+u(t,:);
    end
    
    %% Prepare
    
    all_lambda = nan(n_est,N*n_sim);
    all_param = nan(n_param,n_est+1,n_sim);
    post_var_lambda = zeros(1,N*n_sim);
    
    all_y = Y(2:end-1,:);
    all_x = Y(1:end-2,:);
    all_z = mean(all_y)-rho*mean(all_x);
    
    %% Oracle
    
    i_est = 1;
    
    all_param(:,i_est,:) = repmat([rho; sigma2; ww2_lambda],[1 1 n_sim]);
    
    for i_sim = 1:n_sim
        if mod(i_sim,100) == 0
            disp([i_mix i_est i_sim]);
        end
        samp_ix_xy;
        [all_lambda(i_est,ix),post_var_lambda(ix)] = post_mean_lambda_misspeclik(rho,v_mix(i_mix,:),mmu,ww2_lambda,ddelta);
    end
    
     %% Pooled OLS
     
    i_est = n_est;
    
    for i_sim = 1:n_sim
        samp_ix_xy;
        dX = XX-mean(XX(:));
        rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
        z_rho0;
        all_param(1,i_est,i_sim) = rho0;
        all_lambda(i_est,ix) = mean(z0);
    end
    
    %% Common Parameters: Post Mean (QMLE, Parametric)
    
    i_est = n_est+1;
    
    for i_sim = 1:n_sim
        
        if mod(i_sim,100) == 0
            disp([label ', QMLE, i_sim = ' num2str(i_sim)])
        end
        
        % Initialize empirical Bayesian optimization via Loss-fcn-based result
        samp_ix_xy;
        dX = XX-mean(XX);
        rho0 = (dX(:)'*dX(:))\dX(:)'*YY(:);
        z_rho0;        
        aux = cov(z0,Y0);
        init = [var(YY(:)-XX(:)*rho0-kron(z0',ones(T,1))); aux(1,1)-aux(1,2)^2/aux(2,2)];
             
        % QMLE
        param = fmincon(@n_log_ml_CRE,init,[],[],[],[],lb,ub,[],options);
        sigma2_0 = param(1);
        ww2_lambda0 = param(2);
        
        mX = mean(XX);
        mY = mean(YY);
        
        H = [ones(1,N); Y0];
        MH = eye(N)-(H'/(H*H'))*H;
        
        rho0 = ((XX(:)'*YY(:)-mX*mY'*T)/sigma2_0+mX*MH*mY'/(sigma2_0/T+ww2_lambda0))...
            /((sum(XX(:).^2)-sum(mX.^2)*T)/sigma2_0+sum((mX*MH).^2)/(sigma2_0/T+ww2_lambda0));
        
        z_rho0;
        mmu0 = (H*H')\(H*z0');
        
        all_param(:,i_est,i_sim) = [rho0; sigma2_0; ww2_lambda0];
        
    end
    
    %% Plug-in (QMLE)
    
    i_est_ref = n_est+1;
    i_est = n_est-1;
    plugin;
    
    %% Estimate common parameters for pseudo-out-of-sample forecasts that help with the selection of bhat or Khat 
    
    pseudo_est_CRE;
    
    %% Post Mean (QMLE, Kernel LOO, bhat) 

    i_est_ref = n_est+1;
    i_est = n_est-3;
    basic_kernel_CRE_bhat;
    
    %% Post Mean (QMLE, Mixture, Khat)
    
    i_est_ref = n_est+1;
    i_est = n_est-2;
    init_grp = 1;
    mixture_CRE_Khat;
    
    %% Save
    
    save([temp_path subfolder label '.mat'],'Y','lambda','post_var_lambda','all_param*','all_lambda','all_*hat')
    
    %% Forecast error
    
    all_param(:,n_est+1,:) = [];
    fcst = all_lambda+kron(squeeze(all_param(1,:,:)),ones(1,N)).*Y(end-1,:);
    errs = Y(end,:)-fcst;
    
    % Table
    table_fcst_err_aux = sum(errs.^2,2)/n_sim;    
    table_fcst_err(:,i_mix) = ...
        (table_fcst_err_aux(2:end) - table_fcst_err_aux(1))/(N*mean(post_var_lambda)+N^e0);
    
end

%% Output table

l_col = l_mix;
n_col = n_mix;
l_est = char({'Kernel bhat';'Mixture Khat';'Plug-in';'Pooled OLS'});

fileID = fopen([result_path subfolder 'table.txt'],'w');

fprintf(fileID,'%s',repmat(' ',1,size(l_est,2)));
fprintf(fileID,' %8s',l_col{1:end});
fprintf(fileID,' \r\n');

for i_est = 1:n_est-1
    fprintf(fileID,['%s' repmat(' %8.3f',1,n_col) ' \r\n'],l_est(i_est,:),table_fcst_err(i_est,:));
end

fclose(fileID);